#python

import lx
from math import sqrt

defaultNormalName = 'vert_normals'
tempSetVert = 'vertPrevSelection'
tempSetEdge = 'edgePrevSelection'
tempSetPoly = 'polyPrevSelection'

def getSelectionType():
	selType = lx.eval1('query layerservice selmode ?')
	if selType == 'vertex' or selType == 'edge' or selType == 'polygon':
		return selType
	else:
	 	return 'item'

def vector_length(vec):
	return sqrt(vec[0]**2 + vec[1]**2 + vec[2]**2)

def vector_normalize(vec):
	length = vector_length(vec)
	if length != 0.0:
		return [x / length for x in vec]
	else:
		return [0.0] * 3

def vector_add(vecA, vecB):
	return [a + b for a, b in zip(vecA, vecB)]

def isSP1():
	if lx.eval1('query platformservice appbuild ?') == 59358:
		return True
	else:
		return False

def saveSelection():
	result = [False] * 3

	layers = lx.evalN('query layerservice layers ? fg')
	for layer in layers:
		layerID = lx.eval1('query layerservice layer.ID ? %s' % layer)
		if lx.eval1('query layerservice vert.N ? selected') > 0:
			result[0] = True
		if lx.eval1('query layerservice edge.N ? selected') > 0:
			result[1] = True
		if lx.eval1('query layerservice poly.N ? selected') > 0:
			result[2] = True
		
	if result[0]:
		lx.eval('select.type vertex')
		lx.eval('select.editSet %s add' % tempSetVert)
	if result[1]:
		lx.eval('select.type edge')
		lx.eval('select.editSet %s add' % tempSetEdge)
	if result[2]:
		lx.eval('select.type polygon')
		lx.eval('select.editSet %s add' % tempSetPoly)

	return result

# Select then remove selection set for previously selected elements.
def restoreSelection(selRestore):
	if selRestore[0]:
		lx.eval('select.type vertex')
		lx.eval('!!select.useSet %s select' % tempSetVert)
		lx.eval('vertMap.deleteByName PICK %s' % tempSetVert)
	if selRestore[1]:
		lx.eval('select.type edge')
		lx.eval('!!select.useSet %s select' % tempSetEdge)
		lx.eval('!!vertMap.deleteByName EPCK %s' % tempSetEdge)
	if selRestore[2]:
		lx.eval('select.type polygon')
		lx.eval('!!select.useSet %s select' % tempSetPoly)
		lx.eval('poly.renameTag %s {} pick' % tempSetPoly)

selectionType = getSelectionType()
selRestore = saveSelection()

slowVersion = isSP1()

slowVersion = True

s = lx.Service('layerservice')
s.select('layers', 'main')
mainLayer = s.query1('layers')
mainLayerID = s.query1('layer.ID')

s.select('layers', 'fg')
layers = s.queryN('layers')

vertNormal = [0.0] * 3

layerVerts = {}

for layer in layers:
	s.select('layer.index', '%s' % layer )
	layerID = s.query1('layer.id')

	s.select('verts', 'selected')
	verts = s.queryN('verts')

	thisLayerVerts = []

	for vert in verts:
		s.select('vert.index', vert)
		vlayer = s.query1('vert.layer') + 1
		if vlayer == layer:
			thisLayerVerts.append(vert)
			vnorm = s.queryN('vert.normal')
			vertNormal = vector_add(vertNormal, vnorm)

	layerVerts[layer] = thisLayerVerts

vertNormal = vector_normalize(vertNormal)

for layer in layers:
	s.select('layer.index', '%s' % layer )
	layerID = s.query1('layer.id')
	lx.eval('select.subItem %s set mesh' % layerID)
	lx.eval('select.type vertex')

	try:
		lx.eval('!!select.vertexMap %s norm 3' % defaultNormalName)
	except RuntimeError:
		lx.eval('noop')

	try:
	 	lx.eval('!!vertMap.new %s norm' % defaultNormalName)
	except RuntimeError:
	 	lx.eval('noop')

	try:
		lx.eval('!!select.vertexMap %s norm replace' % defaultNormalName)
	except RuntimeError:
		lx.eval('noop')

	verts = layerVerts[layer]

	for vert in verts:
		#lx.out('Assigning normal to vertex %s of layer %s' % (vert, layer))
		# Put them into the vertex normal map.
		if slowVersion:
			# If we're using 701 SP1, we have to use a slow-ass workaround for broken vertMap.setVertex.
			lx.eval('select.element %s vertex set %s' % (layer, vert))
			lx.eval('vertMap.setValue normal 0 %s' % vertNormal[0])
			lx.eval('vertMap.setValue normal 1 %s' % vertNormal[1])
			lx.eval('vertMap.setValue normal 2 %s' % vertNormal[2])
		else:
			# If we're not using 701 SP1.
			# This way is waaaaaay faster.
			lx.eval('vertMap.setVertex %s normal 0 %s %s' % (defaultNormalName, vert, vertNormal[0]))
			lx.eval('vertMap.setVertex %s normal 1 %s %s' % (defaultNormalName, vert, vertNormal[1]))
			lx.eval('vertMap.setVertex %s normal 2 %s %s' % (defaultNormalName, vert, vertNormal[2]))

		if slowVersion:
			lx.eval('select.drop vertex')

lx.eval('select.subItem %s set mesh' % mainLayerID)
for layer in layers:
	layerID = lx.eval1('query layerservice layer.id ? %s' % layer)
	lx.eval('select.subItem %s add mesh' % layerID)

lx.eval('select.type %s' % selectionType)
restoreSelection(selRestore)